----------------------------------------------------------------------------------
--    This file is part of OsciOne: an open source Android oscilloscope.
--
--    OsciOne is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    OsciOne is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with OsciOne.  If not, see <http://www.gnu.org/licenses/>.
--
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.STD_LOGIC_ARITH.ALL;
--use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity i2c_slave is
    Port ( pSCL : in  STD_LOGIC;
           pSDA : inout  STD_LOGIC;
           clk : in  STD_LOGIC;
		     reset : in STD_LOGIC;
           newDATA : out  STD_LOGIC := '0';
           DATA : out  STD_LOGIC_VECTOR (7 downto 0) := "00000000"
);
end i2c_slave;

architecture Behavioral of i2c_slave is
TYPE state IS (started,receive_data,send_ack, stopped);
signal current_state : state;
signal sr: std_logic_vector(7 DOWNTO 0);
signal count : integer RANGE 0 to 15 := 0;
signal debounce : integer RANGE 0 to 15 := 0;
constant DEB : integer := 7;
constant ACK_count : integer := 8;

signal SDA : std_logic;
signal SCL : std_logic;
signal SDA_old :std_logic;
signal SCL_old : std_logic;
signal drive : boolean := false;

constant I2C_BASE_ADDR : std_logic_vector(7 DOWNTO 4) := "0000";

signal isMyAddr : std_logic := '0';
signal firstByte : std_logic := '0';

begin

clocked: PROCESS(clk, reset)
BEGIN
IF reset = '0' THEN
current_state <= stopped;
SDA <= '0';
SCL <= '0';
newDATA <= '0';
--DATA <= "00000000";
ELSIF rising_edge(clk) THEN

	SDA_old <= SDA;
	SCL_old <= SCL;
	IF SDA /= pSDA OR SCL /= pSCL THEN
	debounce <= debounce + 1;
	ELSE
	debounce <= 0;
	END IF;

	IF debounce=(DEB) THEN
	SDA <= pSDA;
	SCL <= pSCL;
	debounce <= 0;
	END IF;



--find next state
CASE current_state IS

WHEN started =>

IF SCL='0' AND SCL_old='1' THEN --falling edge SCL after START CONDITION
current_state <= receive_data;
ELSE
current_state <= started;
END IF;
firstByte <= '1';
count <= 0;

WHEN receive_data => 

IF count<ACK_count THEN
	IF SCL='1' AND SCL_old='0' THEN -- rising edge SCL, update shift register
	sr <= sr(6 DOWNTO 0) & SDA;
	ELSIF SCL='0' AND SCL_old='1' THEN -- falling edge SCL, update counter
	count <= count + 1;
	ELSIF SCL='1' AND SCL_old='1' AND SDA='1' AND SDA_old='0' THEN -- stop condition
	current_state <= stopped;
	ELSE
	count <= count;
	END IF;
ELSIF count=ACK_count AND firstByte='1' AND sr(7 DOWNTO 4)=I2C_BASE_ADDR AND sr(0)='0' THEN -- master write
firstByte <= '0';
isMyAddr <= '1';
current_state <= send_ack;
count <= 0;
ELSIF count=ACK_count AND isMyAddr='1' THEN
current_state <= send_ack;
count <= 0;
newData <= '1';
--DATA(7 downto 0) <= sr(7 downto 0);

ELSE
current_state <= stopped;
END IF;

WHEN send_ack =>

IF count=0 THEN
drive <= true;
count <= count + 1;
current_state <= send_ack;
ELSIF count=1 THEN
	IF SCL='1' AND SCL_old='0' THEN -- rising edge SCL, we are already sending ACK
	drive <= true;
	current_state <= send_ack;
	count <= count + 1;
	ELSE
	drive <= true;
	current_state <= send_ack;
	count <= count;
	END IF;
ELSIF count=2 THEN
	IF SCL='0' AND SCL_old='1' THEN -- falling edge SCL, release ACK
	drive <= false;
	current_state <= receive_data;
	count <= 0;
	ELSE
	drive <= true;
	current_state <= send_ack;
	count <= count;
	END IF;
ELSE
drive <= true;
current_state <= send_ack;
count <= count;
END IF;
sr <= sr;
newData <= '0';
						

WHEN stopped =>
IF SCL='1' AND SCL_old='1' AND SDA='0' AND SDA_old='1' THEN -- start condition
current_state <= started;
ELSE
current_state <= stopped;
END IF;					
pSDA <= 'Z';
isMyAddr <= '0';
firstByte <= '0';
count <= 0;
newData <= '0';
sr <= "00000000";
END CASE;


END IF;
END PROCESS clocked;

DATA <= sr;
pSDA <= '0' WHEN drive ELSE 'Z';

end Behavioral;
